import yaml


class OpenStackResource:
    def __init__(self, client):
        self.client = client

    def _exec(self, method, *args, yaml_output=True):
        """
        :param method: operation on resource i.e "list", "create"
        :param args: list of string positional args should go first i.e.:
            ["--flavor=m1.tiny"]
            ["vm-test-0", "--flavor=m1.tiny"]
        """
        command = ['openstack', self.resource, method, *args]

        if yaml_output:
            command += ['-f', 'yaml']
            return yaml.safe_load(
                self.client.exec(command))
        return self.client.exec(command)

    def list(self, args):
        return self._exec('list', *args)

    def create(self, args):
        return self._exec('create', *args)

    def delete(self, args, yaml_output=False):
        return self._exec('delete', *args, yaml_output=yaml_output)

    def show(self, args):
        return self._exec('show', *args)

    @property
    def resource(self):
        return self._resource


class Server(OpenStackResource):
    _resource = 'server'


class Flavor(OpenStackResource):
    _resource = 'flavor'


class Stack(OpenStackResource):
    _resource = 'stack'

    def resource_list(self, args, yaml_output=False,):
        return self._exec('resource list',
                          yaml_output=yaml_output,
                          *args)

    def event_list(self, args, yaml_output=False,):
        return self._exec('event list',
                          yaml_output=yaml_output,
                          *args)


class ResourceProvider(OpenStackResource):
    _resource = "resource provider"


class ResourceProviderUsage(OpenStackResource):
    _resource = "resource provider usage"


class Network(OpenStackResource):
    _resource = 'network'


class Hypervisor(OpenStackResource):
    _resource = 'hypervisor'


class Subnet(OpenStackResource):
    _resource = 'subnet'
