import os

from . import commons


class Kubectl:
    def __init__(self, namespace=None, kubeconfig=None):
        self.executor = commons.get_local_executor()
        self.namespace = namespace
        self.result = None
        self.kubeconfig = kubeconfig or os.environ.get("KUBECONFIG")

    def _execute(self, command, **kwargs):
        command = "KUBECONFIG={} kubectl {} ".format(
            self.kubeconfig, command)
        result = self.executor.execute(command, **kwargs)
        commons.LOG.info("%s", result.stdout_str)
        assert result.exit_code == 0, result
        self.result = result
        return self

    def apply(self, apply_file_name="-", stdin=None, cwd=commons.RELEASE_DIR,
              options=""):
        command = "apply -f {} ".format(apply_file_name) + options
        return self._execute(command, stdin=stdin, cwd=cwd)

    def exec(self, pod_name, command, namespace=None):
        namespace = namespace or self.namespace
        command = "exec -n {} {} -- {}".format(namespace, pod_name, command)
        return self._execute(command)

    def get(self, resource_type, command, namespace=None, string=False):
        namespace = namespace or self.namespace
        command = "get {} -n {} {}".format(
            resource_type, namespace, command)
        if string:
            return self._execute(command).result_str
        else:
            return self._execute(command)

    def create(self, resource_type, command, namespace=None):
        namespace = namespace or self.namespace
        command = "create {} -n {} {}".format(
            resource_type, namespace, command)
        return self._execute(command)

    def wait(self, command, namespace=None):
        namespace = namespace or self.namespace
        command = "wait -n {} {}".format(namespace, command)
        return self._execute(command)

    def cp(self, source, destination, container=""):
        container = "-c {}".format(container) if container else container
        command = "cp {} {} {}".format(source, destination, container)
        return self._execute(command)

    def delete(self, resource_type, command, namespace=None, **kwargs):
        namespace = namespace or self.namespace
        command = "delete {} -n {} {}".format(
            resource_type, namespace, command)
        return self._execute(command, **kwargs)

    def get_node_name_by_label(self, label):
        return self.get('nodes',
                        f"-l {label} -ojsonpath='{{.items[*].metadata.name}}'"
                        ).result_str.split()

    @property
    def result_yaml(self):
        return self.result.stdout_yaml

    @property
    def result_str(self):
        return self.result.stdout_str
