import yaml
import json

from si_tests import settings
from si_tests import logger
from si_tests.utils import templates as utils

LOG = logger.logger


class K8SRallyManager(object):
    """Run k8s tests using rally"""

    def __init__(self, kubectl_client, secret_name):
        """kubectl_client: k8s client to Rally testcluster"""
        self._kubectl_client = kubectl_client
        self._secret_name = secret_name

    def run_tests(self, name=settings.RALLY_POD_NAME):
        """Run rally tests"""
        ns = settings.RALLY_NAMESPACE

        options = {
            'RALLY_POD_NAME': name,
            'KUBECONFIG_SECRET_NAME': self._secret_name
        }

        templates = utils.render_template(
            settings.RALLY_POD_YAML, options)
        LOG.debug(templates)
        json_body = json.dumps(yaml.load(templates, Loader=yaml.SafeLoader))

        if self._kubectl_client.pods.present(name=name, namespace=ns):
            LOG.warning("Pod '{0}' already exists. Deleting..".format(name))
            self._kubectl_client.api_core.delete_namespaced_pod(name, ns)

        pod = self._kubectl_client.pods.create(name=name,
                                               namespace=ns,
                                               body=json.loads(json_body))
        pod.wait_phase('Running', timeout=60, interval=10)
        return pod

    def wait_test_results(
            self, pod, timeout=settings.RALLY_WAIT_TIMEOUT):
        pod.wait_test(filepath='./test_*', timeout=timeout)

    def check_test_results(self, pod):
        """Check if the file ./test_passed presents in the pod

        Return True if present, False otherwise
        """
        command = ['/bin/sh', '-c', 'test -f ./test_passed && echo "OK"']
        return pod.exec(command) == "OK\n"

    def download_test_results(self, pod):
        pod.cp_from_pod(source_dir='/rally-reports')
