#    Copyright 2019 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from si_tests import settings
from si_tests import logger

from si_tests.managers import bootstrap_manager
from si_tests.utils import waiters

LOG = logger.logger


class TestProvisionAwsMgmCluster():

    def test_bootstrap_kaas_on_aws_macos_seed_node(self, show_step):
        """Bootstrap KaaS using standalone permanent MacOS seed node

        Scenario:
            1. Verify seed node accessibility
            2. Cleanup seed node docker workspace
               and possible bootstrap leftovers (optional)
            3. Prepare seed node to run KaaS bootstrap on AWS
            4. Check aws bootstrap templates for management cluster (optional)
            5. Bootstrap KaaS with AWS provider
            6. Check versions
            7. Wait for all pods to be Running and Ready
            8. Check that all expected pods exist
            9. Download bootstrap artifacts
            10. Check LDAP integration (optional)
            11. Cleanup seed node docker workspace
                and possible bootstrap leftovers (optional)
        """

        show_step(1)
        assert waiters.icmp_ping(settings.SEED_STANDALONE_EXTERNAL_IP)
        bootstrap = bootstrap_manager.BootstrapManager()

        if settings.KEEP_ENV_BEFORE:
            LOG.info("Skip erase: KEEP_ENV_BEFORE flag is set")
        else:
            show_step(2)
            bootstrap.step_001_cleanup_seed_workspace(None)

        show_step(3)
        bootstrap.step_003_prepare_standalone_seed_node()

        if settings.SKIP_BOOTSTRAP_TEMPLATES_CHECK:
            LOG.warning("Skip bootstrap templates check: "
                        "SKIP_BOOTSTRAP_TEMPLATES_CHECK flag is set")
        else:
            show_step(4)
            bootstrap.check_templates(None)

        show_step(5)
        bootstrap.step_004_deploy_kaas_cluster(None)

        show_step(6)
        bootstrap.step_005_check_versions(None)

        show_step(7)
        bootstrap.step_006_wait_for_pods()

        show_step(8)
        bootstrap.step_006_postdeployment_checks(None)

        show_step(9)
        bootstrap.step_007_download_bootstrap_artifacts(None)

        # need to double check it for standalone seed node
        if settings.KEYCLOAK_LDAP_ENABLED:
            show_step(10)
            bootstrap.check_keystone_ldap_integration()

        if settings.KEEP_ENV_AFTER:
            LOG.warning("Skip erase: KEEP_ENV_AFTER flag is set")
        else:
            # Erase kaas environment after bootstrap is passed
            show_step(11)
            bootstrap.step_007_erase_env_after(None)
            bootstrap.step_001_cleanup_seed_workspace(None)
