from si_tests import settings, logger
from si_tests.managers import bootstrap_manager
import pytest

LOG = logger.logger


def test_get_cluster_logs(kaas_manager, openstack_client,
                          target_clusters_logs):
    """
    Get cluster logs, collected by bootstrap script
    """
    cluster_name = target_clusters_logs.split("/")[1]
    cluster_namespace = target_clusters_logs.split("/")[0]
    ns = kaas_manager.get_namespace(cluster_namespace)
    cluster = ns.get_cluster(cluster_name)

    if not cluster.is_existed():
        pytest.skip("Namespace {0} or cluster {1} doesn't "
                    "exist".format(cluster_namespace, cluster_name))

    bootstrap = bootstrap_manager.BootstrapManager()
    try:
        remote = bootstrap.remote_seed(openstack_client)
    except Exception as e:
        LOG.error(e)
        LOG.error("Cannot connect to seed node. Fallback to "
                  "old collect log mechanism (accessing nodes directly)")
        cluster.private_key = settings.KAAS_PRIVATE_KEY_FILE_PATH
        cluster.key_file_var = 'KAAS_PRIVATE_KEY_FILE_PATH'
        cluster.download_kubernetes_logs(raise_on_error=True)
        cluster.download_system_logs(raise_on_error=True)
        return None

    kubeconfig_path = "kubeconfig"\
        if cluster.is_management else "kubeconfig-{}".format(cluster_name)

    default_key_path = bootstrap.get_bootstrap_ssh_keyfile_path(remote)
    keyfile_path = "id_rsa_{}".format(cluster_name)\
        if settings.KAAS_PRIVATE_KEY_FILE_PATH else default_key_path

    if remote.isdir('./bootstrap/dev'):
        bootstrap_dir = 'bootstrap/dev'
        # bm mgmt - kubeconfig in '~/bootstrap/dev'
        # bm region, mgmt-other - kubeconfig in '~/'
        # bm region, mgmt-bm - kubeconfig in '~/bootstrap/dev'
        # bm child - kubeconfig in '~/bootstrap/dev' or we'll copy it there
        # we rely on either of these 2 folders and kubeconfig format:
        # "kubeconfig-{}".format(cluster_name) for non-mgmt cluster
        # kubeconfig - for mgmt
        kubeconfig_path = kubeconfig_path if remote.isfile(kubeconfig_path) \
            else f'{bootstrap_dir}/{kubeconfig_path}'
        keyfile_path = keyfile_path if remote.isfile(keyfile_path) else \
            f'{bootstrap_dir}/{keyfile_path}'
    else:
        bootstrap_dir = settings.KAAS_BOOTSTRAP_TARGET_DIR

    if not cluster.is_management and \
            not cluster.regional_charts and not remote.isfile(kubeconfig_path):
        LOG.info("Check if child cluster kubeconfig secret exists")
        if cluster.kubeconfig_secret_exists():
            LOG.info("Fetch child cluster kubeconfig secret")
            all_secrets = cluster.get_parent_client().secrets.list(
                namespace=cluster_namespace)
            # We've checked that kubeconfig secret exists earlier
            kubeconfig_secret = [
                x for x in all_secrets
                if x.name == "{}-kubeconfig".format(cluster_name)
            ][0].read()
            LOG.info("Check child cluster kubeconfig in secret is not empty")
            if kubeconfig_secret.data and\
                    'admin.conf' in kubeconfig_secret.data and\
                    kubeconfig_secret.data['admin.conf']:
                _, child_kubeconfig = cluster.get_kubeconfig_from_secret()
                remote.check_call("rm {}|true".format(kubeconfig_path))
                remote.execute("cat << EOF >> {0} \n{1}EOF"
                               .format(kubeconfig_path, child_kubeconfig))
            else:
                LOG.debug("Child cluster kubeconfig is empty:\n{}".format(
                    kubeconfig_secret))
                LOG.warning("Child cluster kubeconfig is empty. Continue..")
                kubeconfig_path = ""
        else:
            LOG.warning("Child cluster kubeconfig was not found. Continue..")
            kubeconfig_path = ""

    if settings.KAAS_PRIVATE_KEY_FILE_PATH:
        LOG.info("Use provided private key")
        with open(settings.KAAS_PRIVATE_KEY_FILE_PATH, 'r') as f:
            id_rsa = f.read()
        remote.check_call("rm {}|true".format(keyfile_path))
        LOG.info("Place private key to seed node to {}".format(keyfile_path))
        remote.execute("cat << EOF >> {0} \n{1}EOF"
                       .format(keyfile_path, id_rsa))
        remote.execute("chmod 400 {}".format(keyfile_path))

    LOG.info("Collect cluster logs ({})".format(cluster_name))
    res = bootstrap.step_collect_logs(remote,
                                      cluster_name,
                                      cluster_namespace,
                                      kubeconfig_path,
                                      keyfile_path,
                                      bootstrap_dir)

    assert res.exit_code == 0,\
        "Error occurred during collecting logs {}".format(res)
